// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef ES_NPU_INTERFACE_H
#define ES_NPU_INTERFACE_H
#include "es_npu_error.h"
#include "es_npu_types.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif /* End of #ifdef __cplusplus */

/**
 * @brief Get the NPU runtime software stack version.
 *
 * @param version: version number.
 * @return error code.
 */
ES_S32 ES_NPU_GetVersion(ES_U64 *version);

/**
 * @brief Gets NPU runtime software stack version information.
 *
 * @param version: runtime version number string.
 * @param maxSize: runtime version number string maximum size.
 * @return error code.
 */
ES_S32 ES_NPU_GetVersionString(ES_CHAR *version, ES_U32 maxSize);

/**
 * @brief Set the log level of the NPU runtime.
 *
 * @param level: log level.
 * @return error code.
 */
ES_S32 ES_NPU_SetLogLevel(ES_U32 level);

/**
 * @brief Get NPU capability information.
 *
 * @param capability: structure to store NPU capability information.
 * @return error code.
 */
ES_S32 ES_NPU_GetCapability(ES_NPU_Capability_S *capability);

/**
 * @brief Get the device properties of the specified deviceId.
 *
 * @param deviceId: device Id.
 * @param prop: device properties to query.
 * @return error code.
 */
ES_S32 ES_NPU_GetDeviceProperties(ES_S32 deviceId, NPU_DEVICE_PROP_S *prop);

/**
 * @brief Get the number of devices available in the current environment.
 *
 * @param deviceNum: the number of devices obtained.
 * @return error code.
 */
ES_S32 ES_NPU_GetNumDevices(ES_U16 *deviceNum);

/**
 * @brief Set device to be used for NPU executions.
 *
 * @param deviceId: device ID of the device wicth to be used.
 * @return error code.
 */
ES_S32 ES_NPU_SetDevice(ES_U16 deviceId);

/**
 * @brief Release the device of the current operation.
 *
 * @param deviceId: device ID which to be released.
 * @return error code.
 */
ES_S32 ES_NPU_ReleaseDevice(ES_U16 deviceId);

/**
 * @brief Get the deviceId in use.
 *
 * @param deviceId: current device ID.
 * @return error code.
 */
ES_S32 ES_NPU_GetDevice(ES_U16 *deviceId);

/**
 * @brief Create a runtime context.
 *
 * @param context: pointer to new context.
 * @param deviceId: device ID of the device which the context associated with.
 * @return error code.
 */
ES_S32 ES_NPU_CreateContext(npu_context *context, ES_U16 deviceId);

/**
 * @brief Set the context on the current thread.
 *
 * @param context The context to be set.
 * @return error code.
 */
ES_S32 ES_NPU_SetCurrentContext(npu_context context);

/**
 * @brief Get the context on the current thread.
 *
 * @param context: pointer to the context.
 * @return error code.
 */
ES_S32 ES_NPU_GetCurrentContext(npu_context *context);

/**
 * @brief Destroys and cleans up a context.
 *
 * @param context: the context to be destroyed.
 * @return error code.
 */
ES_S32 ES_NPU_DestroyContext(npu_context context);

/**
 * @brief Create an asynchronous stream in the context of the current thread.
 *
 * @param stream: pointer to new stream identifier.
 * @return error code.
 */
ES_S32 ES_NPU_CreateStream(npu_stream *stream);

/**
 * @brief Destroys and cleans up an asynchronous stream.
 *
 * @param stream: pointer to new stream identifier.
 * @return error code.
 */
ES_S32 ES_NPU_DestroyStream(npu_stream stream);

/**
 * @brief  Block the thread until all tasks in the specified stream have been completed.
 *
 * @param stream  stream identifier.
 * @return error code.
 */
ES_S32 ES_NPU_SynchronizeStream(npu_stream stream);

/**
 * @brief Abort tasks that are not running in the stream.
 *
 * @param stream: stream identifier.
 * @return error code.
 */
ES_S32 ES_NPU_AbortStream(npu_stream stream);

/**
 * @brief Load offline model data from a file.
 *
 * @param modelId:  model ID returned by the system.
 * @param modelPath: path for offline model file, which includes the file name.
 * @return error code.
 */
ES_S32 ES_NPU_LoadModelFromFile(ES_U32 *modelId, const ES_CHAR *modelPath);

/**
 * @brief Load offline model data from memory.
 *
 * @param modelId:  model ID returned by the system.
 * @param pBuffer: buffer memory for offline model saved.
 * @param nBufLen: the size of buffer memory.
 * @return error code.
 */
ES_S32 ES_NPU_LoadModelFromMemory(ES_U32 *modelId, ES_CHAR *pBuffer, ES_U32 nBufLen);

/**
 * @brief Load Composite model from a specific path.
 *
 * @param modelId:  model ID returned by the system.
 * @param modelPath: path for offline model folder, which includes the models and config file.
 * @return error code.
 */
ES_S32 ES_NPU_LoadCompositeModel(ES_U32 *modelId, const ES_CHAR *modelPath);

/**
 * @brief Get info of composite model by model id.
 *
 * @param modelId:  composite model ID.
 * @param compositeModelInfo: composite model information of modelId return by system.
 * @return error code.
 */
ES_S32 ES_NPU_GetCompositeModelInfo(ES_U32 modelId, NPU_COMPOSITE_MODEL_INFO_S *compositeModelInfo);

/**
 * @brief Unload model from system and cleans up model resources.
 *
 * @param modelId: model ID.
 * @return error code.
 */
ES_S32 ES_NPU_UnloadModel(ES_U32 modelId);

/**
 * @brief Unload composite model from system and cleans up models resources.
 *
 * @param modelId: model ID.
 * @return error code.
 */
ES_S32 ES_NPU_UnloadCompositeModel(ES_U32 modelId);

/**
 * @brief Get the number of input tensors in the model.
 *
 * @param modelId: model ID.
 * @param inputTensors: number of input tensors.
 * @return error code.
 */
ES_S32 ES_NPU_GetNumInputTensors(ES_U32 modelId, ES_S32 *inputTensors);

/**
 * @brief Get the number of output tensors in the model.
 *
 * @param modelId: model ID.
 * @param inputTensors: number of output tensors.
 * @return error code.
 */
ES_S32 ES_NPU_GetNumOutputTensors(ES_U32 modelId, ES_S32 *outputTensors);

/**
 * @brief Get the description of the specified input tensor in the model.
 *
 * @param modelId: model ID.
 * @param tensorId: tensor ID.
 * @param tensor: description of the tensor.
 * @return error code.
 */
ES_S32 ES_NPU_GetInputTensorDesc(ES_U32 modelId, ES_S32 tensorId, NPU_TENSOR_S *tensor);

/**
 * @brief Get the description of the specified outtput tensor in the model.
 *
 * @param modelId: model ID.
 * @param tensorId: tensor ID.
 * @param tensor: description of the tensor.
 * @return error code.
 */
ES_S32 ES_NPU_GetOutputTensorDesc(ES_U32 modelId, ES_S32 tensorId, NPU_TENSOR_S *tensor);

/**
 * @brief Prepares the SMMU mapping on NPU device for the DMA buffer required during task execution.
 * Before calling this interface, it is required to first call ES_NPU_SetDevice.
 *
 * @param devBuf: DMA buffer pointer.
 * @return error code.
 */
ES_S32 ES_NPU_Prepare(ES_DEV_BUF_S *devBuf);

/**
 * @brief Release the SMMU mapping of the DMA Buffer to NPU device and call it in pairs with ES_NPU_Prepare.
 *
 * @param devBuf: DMA buffer pointer.
 * @return error code.
 */
ES_S32 ES_NPU_Unprepare(ES_DEV_BUF_S *devBuf);

/**
 * @brief Submit tasks to SDK, synchronize interfaces, and only return after all tasks are completed.
 *
 * @param tasks: pointer to tasks.
 * @param numTasks: number of tasks.
 * @return error code.
 */
ES_S32 ES_NPU_Submit(NPU_TASK_S *tasks, ES_U32 numTasks);

/**
 * @brief Submit tasks to stream, asynchronize interfaces, immediately return.
 *
 * @param tasks: pointer to tasks.
 * @param numTasks: number of tasks.
 * @param stream: stream to submit.
 * @return error code.
 */
ES_S32 ES_NPU_SubmitAsync(NPU_TASK_S *tasks, ES_U32 numTasks, npu_stream stream);

/**
 * @brief Wait for the specified time, check if there are any tasks completed on the stream,
 *         and then trigger the callback processing of the tasks.
 *
 * @param stream: stream to check.
 * @param timeoutMs: waiting time, in milliseconds. If timeoutMs is -1, wait until a task is completed.
 * @return error code.
 */
ES_S32 ES_NPU_ProcessReport(npu_stream stream, ES_S32 timeoutMs);

/**
 * @brief Set attribute for flexible task.
 *
 * @param modelId: model ID.
 * @param attr: the attribute to set.
 * @return error code.
 */
ES_S32 ES_NPU_SetFlexibleTaskAttr(ES_U32 modelId, NPU_FLEXIBLE_TASK_ATTR_S *attr);

/**
 * @brief Alloc input and output tensor memory of task.
 *
 * @param modelId: model ID.
 * @param nums: numbers of taskMem to allocate.
 * @param taskMem: to save memory allocated.
 * @return error code.
 */
ES_S32 ES_NPU_AllocTaskMemory(ES_U32 modelId, ES_U32 nums, NPU_TASK_MEM_S *taskMem);

/**
 * @brief Alloc input and output tensor memory of task.
 *
 * @param modelId: model ID.
 * @param nums: nums of taskMem to release.
 * @param taskMem: the memory description to release.
 * @return error code.
 */
ES_S32 ES_NPU_ReleaseTaskMemory(ES_U32 modelId, ES_U32 nums, NPU_TASK_MEM_S *taskMem);

/**
 * @brief Submit flexible tasks to stream, asynchronize interfaces.
 *
 * @param tasks: pointer to tasks.
 * @param numTasks: number of tasks.
 * @param stream: stream to submit.
 * @return error code.
 */
ES_S32 ES_NPU_SubmitFlexibleTask(NPU_TASK_S *tasks, ES_U32 numTasks, npu_stream stream);

/**
 * @brief Config dump operator information for model.
 *
 * @param modelId: model ID.
 * @param dumpPath: path to store the dump file.
 * @param dumpConf: configure of dump information.
 * @param dumpConfNum: number of dump configure.
 * @return error code.
 */
ES_S32 ES_NPU_ConfigDumpInfo(ES_U32 modelId, const ES_CHAR *dumpPath, NPU_DUMP_CONF_S *dumpConf, ES_U32 dumpConfNum);

/**
 * @brief DDR read and write size of model.
 *
 * @param modelId: model ID.
 * @param readSize: size of read.
 * @param writeSize: size of write.
 * @return error code.
 */
ES_S32 ES_NPU_GetDdrBandwidth(ES_U32 modelId, ES_U64 *readSize, ES_U64 *writeSize);

/**
 * @brief Enable dump function for model.
 *
 * @param modelId: model ID.
 * @return error code.
 */
ES_S32 ES_NPU_EnableDump(ES_U32 modelId);

/**
 * @brief Disable dump function for model.
 *
 * @param modelId: model ID.
 * @return error code.
 */
ES_S32 ES_NPU_DisableDump(ES_U32 modelId);

/**
 * @brief Return op status for model.
 *
 * @param modelId: model ID.
 * @return error code.
 */
ES_S32 ES_NPU_DumpOpStatus(ES_U32 modelId);

/**
 * @brief Enable performance.
 *
 * @param modelId: model ID.
 * @return error code.
 */
ES_S32 ES_NPU_EnablePerformance(ES_U32 modelId);

/**
 * @brief Disable performance
 *
 * @param modelId: model ID.
 * @return error code.
 */
ES_S32 ES_NPU_DisablePerformance(ES_U32 modelId);

/**
 * @brief Get performance data
 *
 * @param modelId: model ID.
 * @param dir: directory for saving performance data.
 * @return error code.
 */
ES_S32 ES_NPU_DumpPerformanceData(ES_U32 modelId, const ES_CHAR *dir);

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif /* __cplusplus */

#endif
