// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef ESWIN_TYPE_INTERFACE_H
#define ESWIN_TYPE_INTERFACE_H
#include "es_type.h"
#include "es_errno.h"
#include "es_vb_user.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif /* End of #ifdef __cplusplus */

#define ES_NPU_TENSOR_DESC_NAME_MAX_LEN 80U
#define ES_NPU_TENSOR_DESC_NUM_STRIDES 8U
#define ES_NPU_EXCEPTION_INFO_MAX_LEN 256U
#define ES_NPU_TOPOLOGICAL_INFO_MAX_LEN 256U

#define ES_NPU_DATA_FORMAT_UNKNOWN 0U
#define ES_NPU_DATA_FORMAT_NCHW 1U
#define ES_NPU_DATA_FORMAT_NHWC 2U
#define ES_NPU_DATA_FORMAT_NCxHWx 3U

#define ES_NPU_DATA_TYPE_UNKNOWN 0U
#define ES_NPU_DATA_TYPE_FLOAT 1U
#define ES_NPU_DATA_TYPE_HALF 2U
#define ES_NPU_DATA_TYPE_INT16 3U
#define ES_NPU_DATA_TYPE_INT8 4U
#define ES_NPU_DATA_TYPE_UINT8 5U
#define ES_NPU_DATA_TYPE_UINT16 6U
#define ES_NPU_DATA_TYPE_INT32 7U
#define ES_NPU_DATA_TYPE_UINT32 8U
#define ES_NPU_DATA_TYPE_INT64 9U
#define ES_NPU_DATA_TYPE_UINT64 10U

/**
 * @brief Define tensor dimension parameters.
 *
 * @param n: batch size, indicating the number of batches
 * @param c: represents the number of channels in an image
 * @param h: number of pixels representing the vertical dimension of the image
 * @param w: number of pixels representing the horizontal dimension of the image
 */
typedef struct {
    ES_S32 n;
    ES_S32 c;
    ES_S32 h;
    ES_S32 w;
} NPU_DIMS4_S;

/**
 * @brief Define tensor structure parameters.
 *
 * @param name: tensor's name
 * @param bufferSize: tensor buffer size
 * @param dims: tensor dimension parameters
 * @param dataFormat: tensor's dims (NCHW, NHWC, etc.)
 * @param dataType: tensor's data type (int8, int16, fp16, fp32,etc.)
 * @param dataCategory: tensor (image, weight, bias, etc.)
 * @param pixelFormat: pixel format
 * @param pixelMapping: ppixel Mapping
 * @param stride: stride information
 */
typedef struct {
    ES_CHAR name[ES_NPU_TENSOR_DESC_NAME_MAX_LEN + 1];
    ES_U64 bufferSize;
    NPU_DIMS4_S dims;
    ES_U8 dataFormat;
    ES_U8 dataType;
    ES_U8 dataCategory;
    ES_U8 pixelFormat;
    ES_U8 pixelMapping;
    ES_U32 stride[ES_NPU_TENSOR_DESC_NUM_STRIDES];
} NPU_TENSOR_S;

/**
 * @brief The callback function that needs to be called after the task is executed.
 *
 * @param arg: callback function parameter
 */
typedef ES_S32 (*NPU_TaskCallback)(void *arg);

/*
 * @brief Define task state enum.
 */
typedef enum {
    NPU_TASK_STATE_NONE = 0,
    NPU_TASK_STATE_READY = 1,
    NPU_TASK_STATE_RUNNING = 2,
    NPU_TASK_STATE_COMPLETED = 3,
    NPU_TASK_STATE_ABORTED = 4,
    NPU_TASK_STATE_FAILED = 5,
} NPU_TASK_STATE_E;

#define ES_TASK_MAX_FD_CNT 10
#define ES_TASK_SDK_PRIVATE_LEN 64

#define MAX_MODEL_FILE_NAME 512
#define MAX_MODELS_OF_SET 4

/*
 * @brief Define model type enum in model set.
 */
typedef enum {
    MODEL_TYPE_NPU,
    MODEL_TYPE_DSP,
    MODEL_TYPE_GPU,  // reserved
    MODEL_TYPE_INVALID,
} NPU_MODEL_TYPE_E;

/*
 * @brief Define model info description structure params.
 *
 * @param modelName: file name of model
 * @param modelType: model type of model, refer to NPU_MODEL_TYPE_E
 * @param modelCost: cost of model
 * @param modelId: model id of model
 * @param modelDevices: devices number of model used
 */
typedef struct {
    ES_CHAR modelName[MAX_MODEL_FILE_NAME];
    NPU_MODEL_TYPE_E modelType;
    ES_FLOAT modelCost;
    ES_U32 modelId;
    ES_U32 modelDevices;
} NPU_MODEL_INFO_S;

/*
 * @brief Define composite model info description structure params.
 *
 * @param modelNums: number of models in composite model
 * @param modelsInfo: the model info array of models in composite model
 */
typedef struct {
    ES_U32 modelNums;
    NPU_MODEL_INFO_S modelsInfo[MAX_MODELS_OF_SET];
} NPU_COMPOSITE_MODEL_INFO_S;

/*
 * @brief Define flexible task attribute description structure params.
 *
 * @param timeOut: time(ms) for task to wait next.
 */
typedef struct {
    ES_S32 timeOut;
} NPU_FLEXIBLE_TASK_ATTR_S;

/*
 * @brief Define task memory description structure params.
 *
 * @param inputFdNum: number of memory blocks as input
 * @param outputFdNum: number of memory blocks as output
 * @param inputFd: description of memory blocks used as input
 * @param outputFd: description of memory blocks used as output
 */
typedef struct {
    ES_U8 inputFdNum;
    ES_U8 outputFdNum;
    ES_DEV_BUF_S inputFd[ES_TASK_MAX_FD_CNT];
    ES_DEV_BUF_S outputFd[ES_TASK_MAX_FD_CNT];
} NPU_TASK_MEM_S;

/**
 * @brief Define task structure parameters.
 *
 * @param taskId: task ID, assigned by the system after the task is submitted
 * @param modelId: model ID corresponding to the task
 * @param inputFdNum: number of memory blocks as input
 * @param outputFdNum: number of memory blocks as output
 * @param inputFd: description of memory blocks used as input
 * @param outputFd: description of memory blocks used as output
 * @param callback: the callback function that needs to be called after the task is executed
 * @param callbackArg: callback function parameter
 * @param state: status of the task
 * @param sdkPrivate: private field used by SDK
 */
typedef struct {
    ES_U32 taskId;
    ES_U32 modelId;
    ES_U8 inputFdNum;
    ES_U8 outputFdNum;
    ES_DEV_BUF_S inputFd[ES_TASK_MAX_FD_CNT];
    ES_DEV_BUF_S outputFd[ES_TASK_MAX_FD_CNT];
    NPU_TaskCallback callback;
    ES_VOID *callbackArg;
    NPU_TASK_STATE_E state;
    ES_U8 sdkPrivate[ES_TASK_SDK_PRIVATE_LEN];
} NPU_TASK_S;

/**
 * @brief Define the context pointer of npu.
 */
typedef void *npu_context;

/**
 * @brief Define the stream pointer of npu.
 */
typedef void *npu_stream;

typedef struct {
    ES_U32 chipVer;
    ES_U32 memSize;
    ES_U32 isInt8Supported;
    ES_U32 isInt16Supported;
    ES_U32 isInt32Supported;
    ES_U32 isFp16Supported;
    ES_U32 isFp32Supported;
    ES_U32 computingCnt;
    ES_CHAR *topologicalInfo[ES_NPU_TOPOLOGICAL_INFO_MAX_LEN];
} NPU_DEVICE_PROP_S;

/**
 * @brief Define npu dump information.
 *
 * @param subModelId: submodel ID to be dumped
 * @param dumpList: op list that need to be dumped in the submodel
 */
typedef struct {
    ES_U16 subModelId;
    ES_CHAR *dumpList;
} NPU_DUMP_CONF_S;

typedef struct NPU_Capability_S {
    ES_U64 reserved;
} ES_NPU_Capability_S;

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif /* __cplusplus */

#endif
